"use strict";
/*
 * Copyright 2019 Amazon.com, Inc. or its affiliates.
 * Licensed under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EC2Environment = void 0;
const Configuration_1 = __importDefault(require("../config/Configuration"));
const AgentSink_1 = require("../sinks/AgentSink");
const Fetch_1 = require("../utils/Fetch");
const Logger_1 = require("../utils/Logger");
// Documentation for configuring instance metadata can be found here:
// https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/configuring-instance-metadata-service.html
const host = '169.254.169.254';
const tokenPath = '/latest/api/token';
const tokenRequestHeaderKey = 'X-aws-ec2-metadata-token-ttl-seconds';
const tokenRequestHeaderValue = '21600';
const metadataPath = '/latest/dynamic/instance-identity/document';
const metadataRequestTokenHeaderKey = 'X-aws-ec2-metadata-token';
class EC2Environment {
    probe() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const options = {
                    host,
                    path: tokenPath,
                    method: 'PUT',
                    headers: { [tokenRequestHeaderKey]: tokenRequestHeaderValue },
                };
                this.token = yield (0, Fetch_1.fetchString)(options);
            }
            catch (e) {
                (0, Logger_1.LOG)(e);
                return false;
            }
            try {
                const metadataOptions = {
                    host,
                    path: metadataPath,
                    method: 'GET',
                    headers: { [metadataRequestTokenHeaderKey]: this.token },
                };
                this.metadata = yield (0, Fetch_1.fetchJSON)(metadataOptions);
                return !!this.metadata;
            }
            catch (e) {
                (0, Logger_1.LOG)(e);
                return false;
            }
        });
    }
    getName() {
        if (!Configuration_1.default.serviceName) {
            (0, Logger_1.LOG)('Unknown ServiceName.');
            return 'Unknown';
        }
        return Configuration_1.default.serviceName;
    }
    getType() {
        if (this.metadata) {
            return 'AWS::EC2::Instance';
        }
        // this will only happen if probe() is not called first
        return 'Unknown';
    }
    getLogGroupName() {
        return Configuration_1.default.logGroupName ? Configuration_1.default.logGroupName : `${this.getName()}-metrics`;
    }
    configureContext(context) {
        if (this.metadata) {
            context.setProperty('imageId', this.metadata.imageId);
            context.setProperty('instanceId', this.metadata.instanceId);
            context.setProperty('instanceType', this.metadata.instanceType);
            context.setProperty('privateIP', this.metadata.privateIp);
            context.setProperty('availabilityZone', this.metadata.availabilityZone);
        }
    }
    getSink() {
        if (!this.sink) {
            this.sink = new AgentSink_1.AgentSink(this.getLogGroupName(), Configuration_1.default.logStreamName);
        }
        return this.sink;
    }
}
exports.EC2Environment = EC2Environment;
