"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LogSerializer = void 0;
/*
 * Copyright 2019 Amazon.com, Inc. or its affiliates.
 * Licensed under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const heap_1 = require("@datastructures-js/heap");
const Constants_1 = require("../Constants");
const DimensionSetExceededError_1 = require("../exceptions/DimensionSetExceededError");
const StorageResolution_1 = require("../logger/StorageResolution");
/**
 * Serializes the provided context to the CWL Structured
 * Logs format with Embedded Metric Filters.
 */
class LogSerializer {
    /**
     * Retrieve the current context as a JSON string
     */
    serialize(context) {
        const dimensionKeys = [];
        let dimensionProperties = {};
        context.getDimensions().forEach((dimensionSet) => {
            const keys = Object.keys(dimensionSet);
            if (keys.length > Constants_1.Constants.MAX_DIMENSION_SET_SIZE) {
                const errMsg = `Maximum number of dimensions allowed are ${Constants_1.Constants.MAX_DIMENSION_SET_SIZE}.` +
                    `Account for default dimensions if not using set_dimensions.`;
                throw new DimensionSetExceededError_1.DimensionSetExceededError(errMsg);
            }
            dimensionKeys.push(keys);
            dimensionProperties = Object.assign(Object.assign({}, dimensionProperties), dimensionSet);
        });
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const createBody = () => {
            return Object.assign(Object.assign(Object.assign({}, dimensionProperties), context.properties), { _aws: Object.assign(Object.assign({}, context.meta), { CloudWatchMetrics: [
                        {
                            Dimensions: dimensionKeys,
                            Metrics: [],
                            Namespace: context.namespace,
                        },
                    ] }) });
        };
        const eventBatches = [];
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
        let currentBody = createBody();
        // eslint-disable-next-line
        const currentMetricsInBody = () => currentBody._aws.CloudWatchMetrics[0].Metrics.length;
        const hasMaxMetrics = () => currentMetricsInBody() === Constants_1.Constants.MAX_METRICS_PER_EVENT;
        // converts the body to JSON and pushes it into the batches
        const serializeCurrentBody = () => {
            eventBatches.push(JSON.stringify(currentBody));
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            currentBody = createBody();
        };
        const remainingMetrics = heap_1.MaxHeap.heapify(Array.from(context.metrics, ([key, value]) => {
            return { name: key, numLeft: value.values.length };
        }), (metric) => metric.numLeft);
        let processedMetrics = [];
        // Batches the metrics with the most number of values first, such that each metric has no more
        // than 100 values, and each batch has no more than 100 metric definitions.
        while (!remainingMetrics.isEmpty()) {
            const metricProgress = remainingMetrics.extractRoot();
            const metric = context.metrics.get(metricProgress.name);
            if (metric) {
                const startIndex = metric.values.length - metricProgress.numLeft;
                // if there is only one metric value, unwrap it to make querying easier
                const metricValue = metricProgress.numLeft === 1
                    ? metric.values[startIndex]
                    : // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
                        metric.values.slice(startIndex, startIndex + Constants_1.Constants.MAX_VALUES_PER_METRIC);
                // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
                currentBody[metricProgress.name] = metricValue;
                const metricBody = Object.assign({ Name: metricProgress.name, Unit: metric.unit }, (metric.storageResolution == StorageResolution_1.StorageResolution.High ? { StorageResolution: StorageResolution_1.StorageResolution.High } : {}));
                // eslint-disable-next-line
                currentBody._aws.CloudWatchMetrics[0].Metrics.push(metricBody);
                metricProgress.numLeft -= Constants_1.Constants.MAX_VALUES_PER_METRIC;
                if (metricProgress.numLeft > 0) {
                    processedMetrics.push(metricProgress);
                }
                if (hasMaxMetrics() || remainingMetrics.isEmpty()) {
                    serializeCurrentBody();
                    // inserts these metrics back in the heap to be processed in the next iteration.
                    processedMetrics.forEach((processingMetric) => remainingMetrics.insert(processingMetric));
                    processedMetrics = [];
                }
            }
        }
        if (eventBatches.length === 0 || currentMetricsInBody() > 0) {
            serializeCurrentBody();
        }
        return eventBatches;
    }
}
exports.LogSerializer = LogSerializer;
